/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * Random number application
 *
 * Copyright (c) 2005 Lirone Chimoni.  All rights reserved.
 *
 * Lirone Chimoni <lirone@lirone.com>
 *
 *
 * This code is released by the author with no restrictions on usage.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 1.0 $")

#include "asterisk/file.h"
#include "asterisk/app.h"
#include "asterisk/logger.h"
#include "asterisk/options.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"

/* Maximum length of any variable */
#define MAXRESULT	1024

static char *tdesc = "Generate a random number";

static char *app_rand = "Rand";

static char *rand_synopsis = "Return a variable containing a random number";

static char *rand_descrip =
"Usage: Rand(variable|min|limit)\n"
"  variable  - variable name where the random number will be store\n"
"  min       - minimum limit for the random number\n"
"  limit     - maximum interval limit for the random number\n";

STANDARD_LOCAL_USER;

LOCAL_USER_DECL;

static int rand_exec(struct ast_channel *chan, void *data)
{
	int res=0;
	struct localuser *u;
	char *varname, *limit, *min ;
        int x = 0;
        char *argcopy = NULL;
        char *args[3];
        srand( (unsigned)time( NULL ) );
	
	LOCAL_USER_ADD(u);
   
   /* Check and parse arguments */
           if (data) {
                     argcopy = ast_strdupa((char *)data);
     }
           if (ast_separate_app_args(argcopy, '|', args, sizeof(args) / sizeof(args[0])) < 1) 
     {
	
	                ast_log(LOG_WARNING, "Cannot Parse Arguements.\n");
	                return -1;
     }   
           varname = args[x++];
           min = args[x++];
           limit = args[x++];   
	if (varname && min && limit) {
		   sprintf(limit, "%i", ((rand()%atoi(limit))+atoi(min)));
		   pbx_builtin_setvar_helper(chan, varname, limit);
		} else {
			ast_log(LOG_ERROR, "Out of memory\n");
			res = -1;
		}


        LOCAL_USER_REMOVE(u);
	return res;
}

int unload_module(void)
{
	STANDARD_HANGUP_LOCALUSERS;
	return ast_unregister_application(app_rand);
}

int load_module(void)
{
	return ast_register_application(app_rand, rand_exec, rand_synopsis, rand_descrip);
}

char *description(void)
{
	return tdesc;
}

int usecount(void)
{
	int res;
	STANDARD_USECOUNT(res);
	return res;
}

char *key()
{
	return ASTERISK_GPL_KEY;
}
